package java.awt.image.renderable;

import java.awt.geom.AffineTransform;
import java.awt.image.RenderedImage;
import java.awt.RenderingHints;
import java.util.Vector;
import java.awt.geom.Rectangle2D;

public class RenderableImageOp implements RenderableImage
{
    ParameterBlock paramBlock;
    ContextualRenderedImageFactory myCRIF;
    Rectangle2D boundingBox;
    
    public RenderableImageOp(ContextualRenderedImageFactory myCRIF, ParameterBlock parameterBlock) {
        this.myCRIF = myCRIF;
        this.paramBlock = (ParameterBlock)parameterBlock.clone();
    }
    
    @Override
    public Vector<RenderableImage> getSources() {
        return this.getRenderableSources();
    }
    
    private Vector getRenderableSources() {
        Vector<RenderableImage> vector = null;
        if (this.paramBlock.getNumSources() > 0) {
            vector = new Vector();
            int i = 0;
            while (i < this.paramBlock.getNumSources()) {
                Object source = this.paramBlock.getSource(i);
                if ((source instanceof RenderableImage)) {
                    vector.add((RenderableImage)source);
                    ++i;
                }
                else {
                    break;
                }
            }
        }
        return vector;
    }
    
    @Override
    public Object getProperty(String s) {
        return this.myCRIF.getProperty(this.paramBlock, s);
    }
    
    @Override
    public String[] getPropertyNames() {
        return this.myCRIF.getPropertyNames();
    }
    
    @Override
    public boolean isDynamic() {
        return this.myCRIF.isDynamic();
    }
    
    @Override
    public float getWidth() {
        if (this.boundingBox == null) {
            this.boundingBox = this.myCRIF.getBounds2D(this.paramBlock);
        }
        return (float)this.boundingBox.getWidth();
    }
    
    @Override
    public float getHeight() {
        if (this.boundingBox == null) {
            this.boundingBox = this.myCRIF.getBounds2D(this.paramBlock);
        }
        return (float)this.boundingBox.getHeight();
    }
    
    @Override
    public float getMinX() {
        if (this.boundingBox == null) {
            this.boundingBox = this.myCRIF.getBounds2D(this.paramBlock);
        }
        return (float)this.boundingBox.getMinX();
    }
    
    @Override
    public float getMinY() {
        if (this.boundingBox == null) {
            this.boundingBox = this.myCRIF.getBounds2D(this.paramBlock);
        }
        return (float)this.boundingBox.getMinY();
    }
    
    public ParameterBlock setParameterBlock(ParameterBlock parameterBlock) {
        ParameterBlock paramBlock = this.paramBlock;
        this.paramBlock = (ParameterBlock)parameterBlock.clone();
        return paramBlock;
    }
    
    public ParameterBlock getParameterBlock() {
        return this.paramBlock;
    }
    
    @Override
    public RenderedImage createScaledRendering(int n, int n2, RenderingHints renderingHints) {
        double sx = (double)n / (double)this.getWidth();
        double sy = (double)n2 / (double)this.getHeight();
        if (Math.abs(sx / sy - 1.0) < 0.01) {
            sx = sy;
        }
        return this.createRendering(new RenderContext(AffineTransform.getScaleInstance(sx, sy), renderingHints));
    }
    
    @Override
    public RenderedImage createDefaultRendering() {
        return this.createRendering(new RenderContext(new AffineTransform()));
    }
    
    @Override
    public RenderedImage createRendering(RenderContext renderContext) {
        ParameterBlock parameterBlock = (ParameterBlock)this.paramBlock.clone();
        Vector<Object> renderableSources = this.getRenderableSources();
        try {
            if (renderableSources != null) {
                Vector<Object> sources = new Vector();
                int i = 0;
                while (i < renderableSources.size()) {
                    RenderedImage rendering = ((RenderableImage)renderableSources.elementAt(i)).createRendering(this.myCRIF.mapRenderContext(i, renderContext, this.paramBlock, this));
                    if (rendering == null) {
                        return null;
                    }
                    sources.addElement(rendering);
                    ++i;
                }
                if (sources.size() > 0) {
                    parameterBlock.setSources(sources);
                }
            }
            return this.myCRIF.create(renderContext, parameterBlock);
        }
        catch (ArrayIndexOutOfBoundsException ex) {
            return null;
        }
    }
}
